@@ -16,6 +16,9 @@
 
 package org.uberfire.client.workbench.widgets.menu;
 
+import java.util.ArrayList;
+import java.util.List;
+
 import org.jboss.errai.security.shared.api.identity.User;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -30,6 +33,8 @@ import org.uberfire.client.mvp.PerspectiveManager;
 import org.uberfire.client.workbench.events.PerspectiveChange;
 import org.uberfire.client.workbench.events.PlaceMaximizedEvent;
 import org.uberfire.client.workbench.events.PlaceMinimizedEvent;
+import org.uberfire.client.workbench.widgets.menu.events.PerspectiveVisibiltiyChangeEvent;
+import org.uberfire.experimental.service.auth.ExperimentalActivitiesAuthorizationManager;
 import org.uberfire.mvp.Command;
 import org.uberfire.mvp.PlaceRequest;
 import org.uberfire.mvp.impl.DefaultPlaceRequest;
@@ -37,19 +42,33 @@ import org.uberfire.security.Resource;
 import org.uberfire.security.authz.AuthorizationManager;
 import org.uberfire.workbench.model.ActivityResourceType;
 import org.uberfire.workbench.model.menu.MenuFactory;
+import org.uberfire.workbench.model.menu.MenuItem;
 import org.uberfire.workbench.model.menu.MenuPosition;
 import org.uberfire.workbench.model.menu.Menus;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
 import static org.mockito.Matchers.any;
 import static org.mockito.Matchers.anyString;
 import static org.mockito.Matchers.eq;
 import static org.mockito.Matchers.isNull;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.doAnswer;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
 
 @RunWith(MockitoJUnitRunner.class)
 public class WorkbenchMenuBarPresenterTest {
 
+    private static final String NAME = "testName";
+    private static final String PERSPECTIVE_ID = "perspectiveId";
+    private static final String SECOND_PERSPECTIVE_ID = "secondPerspectiveId";
+    private static final String THIRD_PERSPECTIVE_ID = "thirdPerspectiveId";
+
     @Mock
     protected AuthorizationManager authzManager;
     @Mock
@@ -62,14 +81,16 @@ public class WorkbenchMenuBarPresenterTest {
     @Mock
     private WorkbenchMenuBarPresenter.View view;
 
+    @Mock
+    private ExperimentalActivitiesAuthorizationManager experimentalActivitiesAuthorizationManager;
+
     @InjectMocks
     private WorkbenchMenuBarPresenter presenter;
 
     @Test
     public void testAddCurrentPerspective() {
-        final String perspectiveId = "perspectiveId";
-        final Menus menus = MenuFactory.newSimpleItem("test").perspective(perspectiveId).endMenu().build();
-        final PlaceRequest placeRequest = new DefaultPlaceRequest(perspectiveId);
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
+        final PlaceRequest placeRequest = new DefaultPlaceRequest(PERSPECTIVE_ID);
         final PerspectiveActivity perspectiveActivity = mock(PerspectiveActivity.class);
 
         when(perspectiveActivity.getPlace()).thenReturn(placeRequest);
@@ -78,13 +99,12 @@ public class WorkbenchMenuBarPresenterTest {
                                     eq(identity))).thenReturn(true);
 
         presenter.addMenus(menus);
-        verify(view).selectMenuItem(perspectiveId);
+        verify(view).selectMenuItem(PERSPECTIVE_ID);
     }
 
     @Test
     public void testAddPerspective() {
-        final String perspectiveId = "perspectiveId";
-        final Menus menus = MenuFactory.newSimpleItem("test").perspective(perspectiveId).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
         final PlaceRequest placeRequest = new DefaultPlaceRequest("anyId");
         final PerspectiveActivity perspectiveActivity = mock(PerspectiveActivity.class);
 
@@ -96,19 +116,18 @@ public class WorkbenchMenuBarPresenterTest {
         presenter.addMenus(menus);
 
         verify(view,
-               never()).selectMenuItem(perspectiveId);
+               never()).selectMenuItem(PERSPECTIVE_ID);
     }
 
     @Test
     public void testPerspectiveChangeEvent() {
-        final String perspectiveId = "perspectiveId";
-        final Menus menus = MenuFactory.newSimpleItem("test").perspective(perspectiveId).endMenu().build();
-        final PlaceRequest placeRequest = new DefaultPlaceRequest(perspectiveId);
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
+        final PlaceRequest placeRequest = new DefaultPlaceRequest(PERSPECTIVE_ID);
         final PerspectiveActivity perspectiveActivity = mock(PerspectiveActivity.class);
         final PerspectiveChange perspectiveChange = new PerspectiveChange(placeRequest,
                                                                           null,
                                                                           null,
-                                                                          perspectiveId);
+                                                                          PERSPECTIVE_ID);
 
         when(perspectiveActivity.getPlace()).thenReturn(placeRequest);
         when(perspectiveActivity.isType(ActivityResourceType.PERSPECTIVE.name())).thenReturn(true);
@@ -118,14 +137,12 @@ public class WorkbenchMenuBarPresenterTest {
         presenter.addMenus(menus);
         presenter.onPerspectiveChange(perspectiveChange);
 
-        verify(view).selectMenuItem(perspectiveId);
+        verify(view).selectMenuItem(PERSPECTIVE_ID);
     }
 
     @Test
     public void testAddMenuWithPermission() {
-        final String perspectiveId = "perspectiveId";
-        final String label = "perspectiveLabel";
-        final Menus menus = MenuFactory.newSimpleItem(label).perspective(perspectiveId).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
         when(authzManager.authorize(menus.getItems().get(0),
                                     identity)).thenReturn(true);
 
@@ -133,8 +150,8 @@ public class WorkbenchMenuBarPresenterTest {
 
         verify(authzManager).authorize(menus.getItems().get(0),
                                        identity);
-        verify(view).addMenuItem(eq(perspectiveId),
-                                 eq(label),
+        verify(view).addMenuItem(eq(PERSPECTIVE_ID),
+                                 eq(NAME),
                                  isNull(String.class),
                                  any(Command.class),
                                  any(MenuPosition.class));
@@ -142,9 +159,7 @@ public class WorkbenchMenuBarPresenterTest {
 
     @Test
     public void testAddMenuWithoutPermission() {
-        final String perspectiveId = "perspectiveId";
-        final String label = "perspectiveLabel";
-        final Menus menus = MenuFactory.newSimpleItem(label).perspective(perspectiveId).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
         when(authzManager.authorize(menus.getItems().get(0),
                                     identity)).thenReturn(false);
 
@@ -153,8 +168,8 @@ public class WorkbenchMenuBarPresenterTest {
         verify(authzManager).authorize(menus.getItems().get(0),
                                        identity);
         verify(view,
-               never()).addMenuItem(eq(perspectiveId),
-                                    eq(label),
+               never()).addMenuItem(eq(PERSPECTIVE_ID),
+                                    eq(NAME),
                                     isNull(String.class),
                                     any(Command.class),
                                     any(MenuPosition.class));
@@ -162,14 +177,12 @@ public class WorkbenchMenuBarPresenterTest {
 
     @Test
     public void testAddContextMenuWithPermission() {
-        final String perspectiveId = "perspectiveId";
-        final String contextLabel = "contextLabel";
         final MenuPosition position = MenuPosition.LEFT;
-        final Menus contextMenus = MenuFactory.newSimpleItem(contextLabel).endMenu().build();
+        final Menus contextMenus = MenuFactory.newSimpleItem(NAME).endMenu().build();
         final PerspectiveActivity activity = mock(PerspectiveActivity.class);
         final PlaceRequest placeRequest = mock(PlaceRequest.class);
 
-        when(activity.getIdentifier()).thenReturn(perspectiveId);
+        when(activity.getIdentifier()).thenReturn(PERSPECTIVE_ID);
         when(activity.getMenus()).thenReturn(contextMenus);
         when(activity.isType(ActivityResourceType.PERSPECTIVE.name())).thenReturn(true);
         when(authzManager.authorize(contextMenus.getItems().get(0),
@@ -179,14 +192,14 @@ public class WorkbenchMenuBarPresenterTest {
         presenter.onPerspectiveChange(new PerspectiveChange(placeRequest,
                                                             null,
                                                             contextMenus,
-                                                            perspectiveId));
+                                                            PERSPECTIVE_ID));
 
         verify(authzManager).authorize(contextMenus.getItems().get(0),
                                        identity);
         verify(view).clearContextMenu();
-        verify(view).addContextMenuItem(eq(perspectiveId),
+        verify(view).addContextMenuItem(eq(PERSPECTIVE_ID),
                                         anyString(),
-                                        eq(contextLabel),
+                                        eq(NAME),
                                         isNull(String.class),
                                         any(Command.class),
                                         eq(position));
@@ -194,13 +207,11 @@ public class WorkbenchMenuBarPresenterTest {
 
     @Test
     public void testAddContextMenuWithoutPermission() {
-        final String perspectiveId = "perspectiveId";
-        final String contextLabel = "contextLabel";
-        final Menus contextMenus = MenuFactory.newSimpleItem(contextLabel).endMenu().build();
+        final Menus contextMenus = MenuFactory.newSimpleItem(NAME).endMenu().build();
         final PerspectiveActivity activity = mock(PerspectiveActivity.class);
         final PlaceRequest placeRequest = mock(PlaceRequest.class);
 
-        when(activity.getIdentifier()).thenReturn(perspectiveId);
+        when(activity.getIdentifier()).thenReturn(PERSPECTIVE_ID);
         when(activity.getMenus()).thenReturn(contextMenus);
         when(activity.isType(ActivityResourceType.PERSPECTIVE.name())).thenReturn(true);
         when(authzManager.authorize(contextMenus.getItems().get(0),
@@ -210,7 +221,7 @@ public class WorkbenchMenuBarPresenterTest {
         presenter.onPerspectiveChange(new PerspectiveChange(placeRequest,
                                                             null,
                                                             contextMenus,
-                                                            perspectiveId));
+                                                            PERSPECTIVE_ID));
 
         verify(authzManager).authorize(contextMenus.getItems().get(0),
                                        identity);
@@ -226,88 +237,64 @@ public class WorkbenchMenuBarPresenterTest {
 
     @Test
     public void testSetupEnableDisableMenuItemCommand() {
-        final String label = "command";
         final Command command = mock(Command.class);
-        final Menus menus = MenuFactory.newSimpleItem(label).respondsWith(command).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).respondsWith(command).endMenu().build();
 
         when(authzManager.authorize(menus.getItems().get(0),
                                     identity)).thenReturn(true);
 
         presenter.addMenus(menus);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(true);
-        verify(view,
-               times(2)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view, times(2)).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(false);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(false));
+        verify(view).enableMenuItem(anyString(), eq(false));
     }
 
     @Test
     public void testSetupEnableDisableMenuItemPlace() {
-        final String label = "placeLabel";
         final PlaceRequest place = mock(PlaceRequest.class);
-        final Menus menus = MenuFactory.newSimpleItem(label).place(place).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).place(place).endMenu().build();
 
         when(authzManager.authorize(menus.getItems().get(0),
                                     identity)).thenReturn(true);
 
         presenter.addMenus(menus);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(true);
-        verify(view,
-               times(2)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view, times(2)).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(false);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(false));
+        verify(view).enableMenuItem(anyString(), eq(false));
     }
 
     @Test
     public void testSetupEnableDisableMenuItemPerspective() {
-        final String label = "perspectiveLabel";
-        final String perspectiveId = "perspectiveId";
-        final Menus menus = MenuFactory.newSimpleItem(label).perspective(perspectiveId).endMenu().build();
+        final Menus menus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
 
         when(authzManager.authorize(menus.getItems().get(0),
                                     identity)).thenReturn(true);
 
         presenter.addMenus(menus);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(true);
-        verify(view,
-               times(2)).enableMenuItem(anyString(),
-                                        eq(true));
+        verify(view, times(2)).enableMenuItem(anyString(), eq(true));
 
         menus.getItems().get(0).setEnabled(false);
-        verify(view,
-               times(1)).enableMenuItem(anyString(),
-                                        eq(false));
+        verify(view).enableMenuItem(anyString(), eq(false));
     }
 
     @Test
     public void testSetupEnableDisableContextMenuItem() {
-        final String contextLabel = "contextLabel";
-        final String perspectiveId = "perspectiveId";
-        final Menus contextMenus = MenuFactory.newSimpleItem(contextLabel).endMenu().build();
+        final Menus contextMenus = MenuFactory.newSimpleItem(NAME).endMenu().build();
         final PerspectiveActivity activity = mock(PerspectiveActivity.class);
         final PlaceRequest placeRequest = mock(PlaceRequest.class);
 
-        when(activity.getIdentifier()).thenReturn(perspectiveId);
+        when(activity.getIdentifier()).thenReturn(PERSPECTIVE_ID);
         when(activity.getMenus()).thenReturn(contextMenus);
         when(activity.isType(ActivityResourceType.PERSPECTIVE.name())).thenReturn(true);
         when(authzManager.authorize(contextMenus.getItems().get(0),
@@ -317,28 +304,21 @@ public class WorkbenchMenuBarPresenterTest {
         presenter.onPerspectiveChange(new PerspectiveChange(placeRequest,
                                                             null,
                                                             contextMenus,
-                                                            perspectiveId));
-        verify(view,
-               times(1)).enableContextMenuItem(anyString(),
-                                               eq(true));
+                                                            PERSPECTIVE_ID));
+        verify(view).enableContextMenuItem(anyString(), eq(true));
 
         contextMenus.getItems().get(0).setEnabled(true);
-        verify(view,
-               times(2)).enableContextMenuItem(anyString(),
-                                               eq(true));
+        verify(view, times(2)).enableContextMenuItem(anyString(), eq(true));
 
         contextMenus.getItems().get(0).setEnabled(false);
-        verify(view).enableContextMenuItem(anyString(),
-                                           eq(false));
+        verify(view).enableContextMenuItem(anyString(), eq(false));
     }
 
     @Test
     public void testMenuInsertionOrder() {
-        final String perspectiveId = "perspectiveId";
-        final String label = "perspectiveLabel";
-        final Menus firstMenus = MenuFactory.newSimpleItem(label).perspective(perspectiveId).endMenu().build();
-        final Menus secondMenus = MenuFactory.newSimpleItem(label).orderAll(1).perspective(perspectiveId).endMenu().build();
-        final Menus thirdMenus = MenuFactory.newSimpleItem(label).orderAll(2).perspective(perspectiveId).endMenu().build();
+        final Menus firstMenus = MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build();
+        final Menus secondMenus = MenuFactory.newSimpleItem(NAME).orderAll(1).perspective(SECOND_PERSPECTIVE_ID).endMenu().build();
+        final Menus thirdMenus = MenuFactory.newSimpleItem(NAME).orderAll(2).perspective(THIRD_PERSPECTIVE_ID).endMenu().build();
 
         when(authzManager.authorize(firstMenus.getItems().get(0),
                                     identity)).thenReturn(true);
@@ -361,6 +341,43 @@ public class WorkbenchMenuBarPresenterTest {
                    presenter.getAddedMenus().get(2));
     }
 
+    @Test
+    public void testAddMenusWithExperimentalPerspective() {
+
+        when(authzManager.authorize(any(MenuItem.class), any())).thenReturn(true);
+
+        List<MenuItem> items = new ArrayList<>();
+
+        items.add(MenuFactory.newSimpleItem(NAME).perspective(PERSPECTIVE_ID).endMenu().build().getItems().get(0));
+        items.add(MenuFactory.newSimpleItem(NAME).perspective(SECOND_PERSPECTIVE_ID).endMenu().build().getItems().get(0));
+        items.add(MenuFactory.newSimpleItem(NAME).perspective(THIRD_PERSPECTIVE_ID).endMenu().build().getItems().get(0));
+
+        final Menus menus = MenuFactory.newSimpleItem(NAME).withItems(items).endMenu().build();
+
+        when(experimentalActivitiesAuthorizationManager.authorizeActivityId(anyString()))
+                .thenAnswer((Answer<Boolean>) invocationOnMock -> !SECOND_PERSPECTIVE_ID.equals(invocationOnMock.getArguments()[0]));
+
+        presenter.addMenus(menus);
+
+        verify(experimentalActivitiesAuthorizationManager, times(3)).authorizeActivityId(anyString());
+        verify(view, times(2)).setMenuItemVisible(anyString(), eq(true));
+        verify(view).setMenuItemVisible(anyString(), eq(false));
+    }
+
+    @Test
+    public void testShowHideExperimentalPerspectiveMenus() {
+        testAddMenusWithExperimentalPerspective();
+
+        presenter.onPerspectiveVisibilityChange(new PerspectiveVisibiltiyChangeEvent(PERSPECTIVE_ID, false));
+        verify(view).setMenuItemVisible(PERSPECTIVE_ID, false);
+
+        presenter.onPerspectiveVisibilityChange(new PerspectiveVisibiltiyChangeEvent(SECOND_PERSPECTIVE_ID, true));
+        verify(view).setMenuItemVisible(SECOND_PERSPECTIVE_ID, true);
+
+        presenter.onPerspectiveVisibilityChange(new PerspectiveVisibiltiyChangeEvent(THIRD_PERSPECTIVE_ID, false));
+        verify(view).setMenuItemVisible(THIRD_PERSPECTIVE_ID, false);
+    }
+
     @Test
     public void testView() {
         assertEquals(view,
