@@ -22,6 +22,7 @@ import org.uberfire.client.annotations.WorkbenchContext;
 import org.uberfire.client.annotations.WorkbenchPopup;
 import org.uberfire.client.annotations.WorkbenchScreen;
 import org.uberfire.client.annotations.WorkbenchSplashScreen;
+import org.uberfire.experimental.definition.annotations.ExperimentalFeature;
 
 /**
  * A non-functional class that exists only to support the UF-44 regression test.
@@ -32,6 +33,7 @@ import org.uberfire.client.annotations.WorkbenchSplashScreen;
 @WorkbenchPopup(identifier = "sample")
 @WorkbenchScreen(identifier = "sample")
 @WorkbenchSplashScreen(identifier = "sample")
+@ExperimentalFeature
 public class AnnotatedWithEverything {
 
 }
